@@ -1,4 +1,4 @@
-from ruleskit import Activation, Rule, RegressionRule, ClassificationRule, RuleSet
+from ruleskit import Activation, Rule, RegressionRule, ClassificationRule, RuleSet, Thresholds
 import pytest
 
 
@@ -16,6 +16,16 @@ def clean_for_stacked_fit():
     RuleSet.STACKED_FIT = False
 
 
+@pytest.fixture
+def clean_for_stacked_fit_th():
+    RuleSet.STACKED_FIT = True
+    yield
+    Activation.clean_files()
+    RuleSet.STACKED_FIT = False
+    ClassificationRule.SET_THRESHOLDS(None)
+    RegressionRule.SET_THRESHOLDS(None)
+
+
 @pytest.fixture
 def prepare_fit():
     calc_attributes_r = Rule.calc_attributes
